import BuildControls from '@/components/BuildControls/BuildControls';
import Burger from '@/components/Burger/Burger';
import OrderSummary from '@/components/Burger/OrderSummary/OrderSummary';
import Modal from '@/components/UI/Modal/Modal';
import { IngredientNames, IngredientPrices, Ingredients } from '@/interfaces/Ingredients';
import { useState } from 'react';
import { createSearchParams, URLSearchParamsInit, useNavigate, useSearchParams } from 'react-router-dom'

const BurgerBuilder = () => {
    const navigate = useNavigate()

    const [searchParams, setSearchParams] = useSearchParams();
    const [totalPrice, setTotalPrice] = useState<number>(IngredientPrices.bread);
    const [purchasable, setPurchasable] = useState<boolean>(false);
    const [open, setOpen] = useState<boolean>(false);
    const [ingredients, setIngredients] = useState<Ingredients>({
        salad: 0,
        cheese: 0,
        chicken: 0,
        meat: 0
    });

    const updatePurschasable = (ingredients: Ingredients) => {
        const sum = Object.keys(ingredients)
                            .map(ingKey => ingredients[ingKey as IngredientNames])
                            .reduce((sum, el) => sum + el, 0)

        setPurchasable(sum > 0)
    }

    const moreHandler = (ingKey: IngredientNames) => {
        const oldCount = ingredients[ingKey];
        const updatedCount = oldCount + 1;
        const updatedIngredients = { ...ingredients };
        updatedIngredients[ingKey] = updatedCount;
        setIngredients(updatedIngredients);

        const priceAddition = IngredientPrices[ingKey];
        const newPrice = totalPrice + priceAddition;
        setTotalPrice(newPrice);

        updatePurschasable(updatedIngredients);
    }

    const lessHandler = (ingKey: IngredientNames) => {
        const oldCount = ingredients[ingKey];
        if (oldCount <= 0) {
            return;
        }

        const updatedCount = oldCount - 1;
        const updatedIngredients = { ...ingredients };
        updatedIngredients[ingKey] = updatedCount;
        setIngredients(updatedIngredients);

        const priceAddition = IngredientPrices[ingKey];
        const newPrice = totalPrice - priceAddition;
        setTotalPrice(newPrice);

        updatePurschasable(updatedIngredients);
    }

    const onContinueHandler = () => {
        const params = createSearchParams(ingredients as unknown as URLSearchParamsInit)

        setSearchParams(params)
        navigate('/checkout')
    }

    return (
        <>
            <Modal open={open} setOpen={setOpen}>
                <OrderSummary 
                    price={totalPrice} 
                    ingredients={ingredients}
                    onClick={onContinueHandler}
                />
            </Modal>

            <Burger ingredients={ingredients}/>
            <BuildControls 
                ingredients={ingredients} 
                lessHandler={lessHandler}
                moreHandler={moreHandler}
                setOpen={setOpen}
                price={totalPrice}
                purchasable={purchasable}
            />
        </>
    )
}

export default BurgerBuilder;
